C	raw2sum.for
C	10-JUL-1995, David Mathog, Biology division, Caltech
C
C	Reads a raw "minute" records file, sums for each user, and
C	appends that to a "sum" file.  The user name must be in the
C	first 8 positions of each record - the rest of the line is
C	ignored.
C
C	RAWMINUTE	Environmental variable or filename for 
C	                the raw minutes file.
C	SUMMINUTE	Environmental variable or filename for
C			the sum minutes file.
C
C	************************************************************
	integer maxusers
	parameter (maxusers = 1000)
	character*8 users(maxusers)
	character*8 line
	integer     usage(maxusers)
	integer     istat,found,lasthit
	logical     ok,new
C
C	Open input/output files
C
	open(unit=10,iostat=istat,file='RAWMINUTE'
	1   ,form='FORMATTED'
	1   ,carriagecontrol='LIST',access='SEQUENTIAL'
	1   ,status='OLD',READONLY)
	if(istat.ne.0)stop 'Fatal error:RAWMINUTE would not open'
C
	open(unit=11,iostat=istat,file='SUMMINUTE'
	1   ,form='FORMATTED',status='UNKNOWN'
	1   ,access='APPEND',carriagecontrol='LIST')
	if(istat.ne.0)stop 'Fatal error:SUMMINUTE would not open'
C
C	initialize a couple of variables
C
	ok      = .true.
	found   = 0
	lasthit = 0
C
C	zero out all registers
C
	do i = 1,maxusers
	  users(i)='        '
	  usage(i)=0
	end do
C
C	scan all records
C
C	Note that when checking for a matching name, always check the
C	last used name first, this will be a match 99% of the time.
C	Only if that fails do we check others.
C
1000	format(a8)
	do while(ok)
	   read(10,1000,iostat=istat)line
	   if(istat .eq. 0)then
	     new = .true.
	     if(found .ge. 1)then
	        if(users(lasthit) .eq. line)then
	          usage(found)=usage(found) + 1
	          new = .false.
	        else
	          i = 1
	          do while(i .le. found)
	            if(users(i) .eq. line)then
	               usage(i)=usage(i) + 1
	               lasthit = i
	               new = .false.
	               i = found + 1
	            end if
	            i = i + 1
	          end do
	        end if
	     end if	      
	     if(new)then
	        found = found + 1
	        users(found)=line
	        usage(found)=1
	        lasthit=found
	     end if
	   else
	      ok = .false.
	   end if
	end do
C
C	At this point all lines have been processed, write out
C	the results
C
2000	format(a8,i9)
	do i = 1,found
	  write(11,2000)users(i),usage(i)
	end do
C
C	close the files
C
	close(10,STATUS='KEEP')
	close(11,STATUS='KEEP')
C
	stop 'raw2sum: processing completed'
	end
